// GlobalStat, file gsactivebar.cpp
//
// Copyright  1997 Olivier Wittenberg

#include "gsactivebar.h"
#include "../../Sources/Public/GSActiveView.h"
#include "../../Sources/Public/GClient.h"

const float height = 25;

//
// Constructor
//

GSActiveBar::GSActiveBar (BRect frame) : GSActiveView (frame, B_WILL_DRAW), oldx (0)
{
	// A gray background
	SetLowColor (230, 230, 230);
}


//
// StrokeBiColorRect method ('b' indicates which color should be used on the corners)
//

void GSActiveBar::StrokeBiColorRect (BRect r, rgb_color c1, rgb_color c2, bool b)
{
	AddLine (BPoint (r.left, r.top), BPoint (b ? r.right - 1 : r.right, r.top), c1);
	AddLine (BPoint (r.left, r.top + 1), BPoint (r.left, b ? r.bottom - 1 : r.bottom), c1);
	AddLine (BPoint (b ? r.left : r.left + 1, r.bottom), BPoint (r.right, r.bottom), c2);
	AddLine (BPoint (r.right, b ? r.top : r.top + 1), BPoint (r.right, r.bottom - 1), c2);
}


//
// Draw method
//

void GSActiveBar::Draw (BRect)
{
	// Constants
	const rgb_color bluebar   = {51, 152, 255},
					lightblue = {152, 203, 255},
					darkblue  = {0, 102, 152},
					white	  = {255, 255, 255},
					grey	  = {200, 200, 200},
					darkwhite = {250, 250, 250},
					darkgrey  = {80, 80, 80},
					midgrey	  = {120, 120, 120};

	float ratio = (CtDatatype () == GS_TYPE_FLOAT ? CtValueF () : CtValueL ()) / CtMax ();

	BRect rectframe = Bounds ();
	// Compute the new frame for GSActiveBar
	float offset = (rectframe.bottom - rectframe.top + 1 - height) / 2;
	rectframe.left  += 5;
	rectframe.right -= 5;
	rectframe.top 	 += offset;
	rectframe.bottom -= offset;

	{
		// We'll use a clipping region to avoid blinking
		BRegion clipreg;
	
		// If SetViewColor was used, there would be a gray blinking on the bar, regardless of the clipping region
		clipreg.Include (Bounds ());
		clipreg.Exclude (rectframe);
		ConstrainClippingRegion (&clipreg);
		FillRect (Bounds (), B_SOLID_LOW);
	}

	float newx = rectframe.left + (rectframe.right - rectframe.left) * ratio;

	SetHighColor (bluebar);
	FillRect (BRect (rectframe.left + 3, rectframe.top + 3, newx - 1, rectframe.bottom - 3));

	SetHighColor (white);
	FillRect (BRect (newx + 1, rectframe.top + 2, rectframe.right - 2, rectframe.bottom - 2));

	BeginLineArray (12);

	StrokeBiColorRect (
		BRect (rectframe.left + 2, rectframe.top + 2, newx, rectframe.bottom - 2),
		lightblue,
		darkblue,
		FALSE
	);

	StrokeBiColorRect (rectframe, grey, darkwhite, TRUE);
	StrokeBiColorRect (
		BRect (rectframe.left + 1, rectframe.top + 1, rectframe.right - 1, rectframe.bottom - 1),
		darkgrey,
		midgrey,
		TRUE
	);

	EndLineArray ();

	oldx = newx;
}


//
// NewValue: this method is overloaded because we don't need to invalidate the whole view
//

void GSActiveBar::NewValue (bool has_changed)
{
	if (has_changed) {
		BRect rectframe = Bounds ();

		float offset = (rectframe.bottom - rectframe.top + 1 - height) / 2;
		rectframe.left  += 7;
		rectframe.right -= 7;
		rectframe.top 	 += offset + 2;
		rectframe.bottom -= offset + 2;

		Invalidate (rectframe);
	}
}


//
// Global symbols
//

#pragma export on
GSActiveBar *global_constructor (float, float, float, float);
float view_minimum_height = height + 6;
#pragma export reset

GSActiveBar *global_constructor (float x1, float y1, float x2, float y2)
{
	return (new GSActiveBar (BRect (x1, y1, x2, y2)));
}