// GlobalStat, file gsactivegraph.cpp
//
// Copyright  1997 Olivier Wittenberg

#include "gsactivegraph.h"
#include "../../Sources/Public/GClient.h"
#include "../../Sources/Public/GSActiveView.h"

const float height = 50;

//
// Constructor
//

GSActiveGraph::GSActiveGraph (BRect frame) : GSActiveView (frame, B_WILL_DRAW), nvalues (0)
{
	// A grey background
	SetViewColor (230, 230, 230);
}


//
// StrokeBiColorRect method ('b' indicates which color should be used on the corners)
//

void GSActiveGraph::StrokeBiColorRect (BRect r, rgb_color c1, rgb_color c2, bool b)
{
	SetHighColor (c1);
	StrokeLine (BPoint (r.left, r.top), BPoint (b ? r.right - 1 : r.right, r.top));
	StrokeLine (BPoint (r.left, r.top + 1), BPoint (r.left, b ? r.bottom - 1 : r.bottom));

	SetHighColor (c2);
	StrokeLine (BPoint (b ? r.left : r.left + 1, r.bottom), BPoint (r.right, r.bottom));
	StrokeLine (BPoint (r.right, b ? r.top : r.top + 1), BPoint (r.right, r.bottom - 1));
}


//
// Draw method
//

void GSActiveGraph::Draw (BRect)
{
	const rgb_color	grey	  = {200, 200, 200},
					darkwhite = {250, 250, 250},
					darkgrey  = {80, 80, 80},
					midgrey	  = {120, 120, 120},
					black     = {0, 0, 0};

	BRect rectframe = Bounds ();

	rectframe.left  += 5;
	rectframe.right -= 5;
	rectframe.top 	 += 5;
	rectframe.bottom -= 5;

	float unitx = (rectframe.Width () - 4) / (REMEMBER_N - 1);
	float ratio = (rectframe.Height () - 4) / CtMax ();

	StrokeBiColorRect (rectframe, grey, darkwhite, TRUE);
	StrokeBiColorRect (
		BRect (rectframe.left + 1, rectframe.top + 1, rectframe.right - 1, rectframe.bottom - 1),
		darkgrey,
		midgrey,
		TRUE
	);

	float curx;

	SetHighColor (black);
	MovePenTo (curx = 7, rectframe.bottom - 2 - values[0] * ratio);

	for (int i = 1; i < nvalues; i++)
		StrokeLine (BPoint (curx += unitx, rectframe.bottom - 2 - values[i] * ratio));
}


//
// NewValue hook
//

void GSActiveGraph::NewValue (bool)
{
	float newvalue = CtDatatype () == GS_TYPE_FLOAT ? CtValueF () : CtValueL ();

	if (nvalues < REMEMBER_N)
		// Add the new value to the list
		values[nvalues++] = newvalue;
	else {
		// Shift the values to the left
		for (int i = 1; i < REMEMBER_N; i++)
			values[i - 1] = values[i];

		values[REMEMBER_N - 1] = newvalue;
	}

	// No need to invalidate the whole view
	Invalidate (BRect (7, 7, Bounds ().right - 7, Bounds ().bottom - 7));
}


//
// Global symbols
//

#pragma export on
GSActiveGraph *global_constructor (float, float, float, float);
float view_minimum_height = height + 6;
#pragma export reset

GSActiveGraph *global_constructor (float x1, float y1, float x2, float y2)
{
	return (new GSActiveGraph (BRect (x1, y1, x2, y2)));
}