// CPUClient for GlobalStat
//
// Copyright  1997 Olivier Wittenberg

#include "CPUApp.h"
#include "CPUClient.h"

main ()
{
	if (CPUApp *c = new CPUApp) {
		c->Run ();
		delete c;
	}
	return (0);
}


//
// Constructor: create the clients
//

CPUApp::CPUApp () : BApplication ('Gcpu')
{
	char title[] = "CPU n load";
	system_info si;

	// We need some information
	get_system_info (&si);
	cpu_count = si.cpu_count;

	bool at_least_one = FALSE;

	// Initialize last_active_times[] and create the clients
	for (long i = 0; i < cpu_count; i++) {
		title[4] = i + '1';

		if (clients[i] = new CPUClient (this, title, title, i))
			if (int err = clients[i]->Error ()) {
				const char *errtxt[] = {
					NULL,
					"This program requires GlobalStat",
					"Insufficient memory",
					"Invalid reply from GlobalStat !",
					"GlobalStat doesn't reply",
					"Cannot send to GlobalStat",
					"GClient: wrong parameters !"
				};

				if (BAlert *a = new BAlert ("alert", errtxt[err], "OK"))
					a->Go ();

				cpu_count = i + 1; // See the destructor
				PostMessage (B_QUIT_REQUESTED);
				return;
			}
			else
				at_least_one = TRUE;
	}

	if (!at_least_one)
		PostMessage (B_QUIT_REQUESTED);
	else
		SetPulseRate (2000000);
}


//
// Destructor: delete the clients
//

CPUApp::~CPUApp ()
{
	for (long i = 0; i < cpu_count; i++)
		if (clients[i])
			delete clients[i];
}


//
// Pulse: send data
//

void CPUApp::Pulse ()
{
	for (long i = 0; i < cpu_count; i++)
		if (clients[i])
			clients[i]->SendCPULoad ();
}