// LoginWatch Client for GlobalStat
//
// Copyright  1997 Olivier Wittenberg

#include "../../Sources/Public/GClient.h"
#include "LWApp.h"

//
// main: as usual
//

main ()
{
	if (LWApp *c = new LWApp) {
		c->Run ();
		delete c;
	}
	return (0);
}


//
// Constructor
//

LWApp::LWApp () : BApplication ('GLgW')
{
	// Create the list that will hold ClientItem objects
	if (!(clients = new BList)) {
		be_app->PostMessage (B_QUIT_REQUESTED);
		return;
	}

	// A hidden window so that the dock menu is reachable
	if (!new BWindow (BRect (10, 10, 20, 20), "Quit w/dock", B_MODAL_WINDOW, 0)) {
		PostMessage (B_QUIT_REQUESTED);
		return;
	}

	// Start pulsing...
	SetPulseRate (2000000);

	// ... as soon as possible
	PostMessage (B_PULSE);
}


//
// Destructor: delete the clients
//

LWApp::~LWApp ()
{
	if (clients) {
		long m = clients->CountItems ();

		// The ClientItem object automatically deletes its GClient object
		for (long i = 0; i < m; i++)
			delete (ClientItem *)clients->ItemAt (i);

		delete clients;
	}
}


//
// Pulse: create, update, and delete clients
//

void LWApp::Pulse ()
{
	// First, set the "still_exists" flag to FALSE, for every client
	{
		long max = clients->CountItems ();

		while (max--)
			((ClientItem *)clients->ItemAt (max))->still_exists = FALSE;
	};

	team_info	tinf;
	client_type ctype;

	for (long n = 0; get_nth_team_info (n, &tinf) == B_NO_ERROR; n++) {
		ctype = strncmp (tinf.args, "ftpd -a ", 8) ?
				strncmp (tinf.args, "telnetd -a ", 11) ?
				LW_INVALID : LW_TELNET : LW_FTP;

		if (ctype != LW_INVALID) {
			// Is it a new ftp or telnet ?
			long max = clients->CountItems (), i;

			for (i = 0; i < max; i++) {
				ClientItem *ci = (ClientItem *)clients->ItemAt (i);

				if (ci->watched_team == tinf.team) {
					// We've found it, so update it
printf("UPDATE TEAM %ld THREAD %ld\n",ci->watched_team,ci->watched_thread);
					ci->still_exists = TRUE;

					thread_info ti;
					get_thread_info (ci->watched_thread, &ti);
					ci->client->SendData (ti.user_time - ci->last_user_time);
					ci->last_user_time = ti.user_time;
					break;
				}
			}

			if (i == max) {
				// It's new !
				char uname[16], title[128];

				// We need a unique name: let's sprintf the team id:
				sprintf (uname, "%ld", tinf.team);

				// The title will look like "FTP from 165.134.1.5"
				strcpy (title, ctype == LW_FTP ? "FTP" : "Telnet");
				strcat (title, " from ");
				strcat (title, tinf.args + (ctype == LW_FTP ? 8 : 11));

				// Cut the port number (args looks like "ftpd -a 165.134.1.5 21")
				if (char *s = strchr (title + (ctype == LW_FTP ? 9 : 12), ' '))
					*s = 0;

				// Create the client and connect to GlobalStat
				GClient	   *gc = new GClient (this, uname, title, NULL, GS_TYPE_FLOAT);
				ClientItem *ci = NULL;

				// Error checking
				if (gc)
					if (int err = gc->Error ()) {
						const char *errtxt[] = {
							NULL,
							"This program requires GlobalStat",
							"Insufficient memory",
							"Invalid reply from GlobalStat !",
							"GlobalStat doesn't reply",
							"Cannot send to GlobalStat",
							"GClient: wrong parameters !"
						};

						if (BAlert *a = new BAlert ("alert", errtxt[err], "OK"))
							a->Go ();

						PostMessage (B_QUIT_REQUESTED);
						delete gc;
					}
					else
						ci = new ClientItem;

				if (!ci) {
					if (gc)
						delete gc;
				}
				else {
					// Everything went ok
					thread_info ti;
					get_nth_thread_info (tinf.team, ctype == LW_FTP ? 0 : 2, &ti);

					ci->client			= gc;
					ci->watched_thread	= ti.thread;
					ci->watched_team	= tinf.team;
					ci->still_exists	= TRUE;
					ci->last_user_time	= ti.user_time;

					clients->AddItem (ci);
printf("NEW !  TEAM %ld THREAD %ld\n",ci->watched_team,ci->watched_thread);
				}
			}
		}
	}

	// Delete the dead clients
	long max = clients->CountItems ();

	for (long i = 0; i < max; i++) {
		ClientItem *ci = (ClientItem *)clients->ItemAt (i);

		if (!ci->still_exists) {
			// Remove and delete it
			clients->RemoveItem (i--);
			max--;

printf("DEAD ! TEAM %ld THREAD %ld\n",ci->watched_team,ci->watched_thread);

			delete ci;
		}
	}
}


//
// AboutRequested
//

void LWApp::AboutRequested ()
{
	if (BAlert *a = new BAlert ("About",
		"LoginWatch client for GlobalStat\n\n"
		"This client shows the activity of incoming ftp and telnet sessions. "
		"Use the dock menu to quit.\n\n"
		" 1997 Olivier Wittenberg, witt@lenet.fr",
		"OK"
	))
		a->Go ();
}